@@ -5,6 +5,7 @@
 # TODO: (FEAT) support for multiple transports on same run
 # TODO: (FEAT) support for different statistical distribution
 # TODO: (FEAT) support batch mode
+# TODO: support throughput testing thru http://linux.die.net/man/1/pv
 
 
 import pylog_config
@@ -19,6 +20,9 @@ import datetime
 import transports as trans
 import formatters as forms
 
+# import fake factory
+from faker import Factory
+
 
 DEFAULT_BASE_LOGGING_LEVEL = logging.INFO
 DEFAULT_VERBOSE_LOGGING_LEVEL = logging.DEBUG
@@ -139,7 +143,7 @@ def calculate_throughput(elapsed_time, messages):
         return 'Unknown', seconds
 
 
-def send(instance, client, format, format_config, messages, gap):
+def send(instance, client, format, format_config, messages, gap, batch):
     """sends logs and prints the time it took to send all logs
 
     :param instance: transport class instance
@@ -148,6 +152,7 @@ def send(instance, client, format, format_config, messages, gap):
     :param dict format_config: formatter configuration to use
     :param int messages: number of messages to send
     :param float gap: gap in seconds between 2 messages
+    :param int batch: number of messages per batch
     """
     message_count = 0
     lgr.debug('configuring formatter...')
@@ -158,10 +163,11 @@ def send(instance, client, format, format_config, messages, gap):
     lgr.debug('start time is: {}'.format(start_time))
     while True:
         # generate the log data from the formatter
-        log = formatter_instance.generate_data()
+        logs = [formatter_instance.generate_data() for i in xrange(batch)]
         # and send the log through the relevant transport
-        instance.send(client, log)
-        message_count += 1
+        for log in logs:
+            instance.send(client, log)
+        message_count += batch
         # check if the number of messages sent are less than the desired amount
         if message_count < messages:
             # just to get some feedback during execution
@@ -170,22 +176,31 @@ def send(instance, client, format, format_config, messages, gap):
             # and sleep the desired amount of time.. zzz zz zZZ zZZzzzz
             sleep(gap)
         else:
-            # then get the current time once more
-            end_time = get_current_time()
-            lgr.debug('end time is: {}'.format(end_time))
-            # and the elapsed time
-            elapsed_time = end_time - start_time
-            # meH!
-            throughput, seconds = calculate_throughput(elapsed_time, messages)
-            # TODO: add the option to send the throughput as well to benchmark
-            # TODO: the logging process itself.
-            lgr.info('DONE! (after {}h ({} seconds) with '
-                     'throughput: {} logs/sec)'.format(
-                         elapsed_time, seconds, throughput))
-            return
+            break
+    # then get the current time once more
+    end_time = get_current_time()
+    lgr.debug('end time is: {}'.format(end_time))
+    # and the elapsed time
+    elapsed_time = end_time - start_time
+    # meH!
+    throughput, seconds = calculate_throughput(elapsed_time, messages)
+    # TODO: add the option to send the throughput as well to benchmark
+    # TODO: the logging process itself.
+    lgr.info('DONE! (after {}h ({} seconds) with '
+             'throughput: {} logs/sec)'.format(
+                 elapsed_time, seconds, throughput))
+    # TODO: support retrieving data statistics from transport after
+    # TODO: execution ended.
+    try:
+        data = instance.get_data()
+    except AttributeError:
+        lgr.warning(
+            'statistical data not implemented for chosen transport.')
+    lgr.info('statistical data: {}'.format(data))
+    return
 
 
-def config_transport(transport, transport_config):
+def config_transport(transports, transport, transport_config):
     """returns a configured instance and client for the transport
 
     :param string transport: transport to use
@@ -193,14 +208,17 @@ def config_transport(transport, transport_config):
     """
     lgr.debug('configuring transport...')
     # get transport instance
-    instance = getattr(trans, transport)(transport_config)
+    try:
+        instance = getattr(transports, transport)(transport_config)
+    except AttributeError:
+        raise PylogError('missing transport')
     # get logging client
     client = instance.configure()
     return instance, client
 
 
 def generator(config=None, transport=None, formatter=None, gap=None,
-              messages=None, verbose=False):
+              messages=None, batch=False, verbose=False):
     """generates log messages
 
     this will generate log message in the requested format and protocol.
@@ -233,6 +251,9 @@ def generator(config=None, transport=None, formatter=None, gap=None,
     if not messages:
         messages = DEFAULT_NUMBER_OF_MESSAGES
         lgr.info('message count not specified. assuming: {}'.format(messages))
+    batch = int(batch) if batch else 1
+    if batch > int(messages):
+        raise PylogError('batch number larger than total amount of messages')
     # set verbosity level for internal logging
     set_global_verbosity_level(verbose)
     # import config file with formatter and transport config
@@ -249,10 +270,10 @@ def generator(config=None, transport=None, formatter=None, gap=None,
         raise PylogError(
             'formatter configuration not specified for {}'.format(formatter))
     # configure transport instance and client
-    instance, client = config_transport(transport, transport_config)
+    instance, client = config_transport(trans, transport, transport_config)
     # log messages
     send(instance, client, formatter, formatter_config,
-         int(messages), float(gap))
+         int(messages), float(gap), batch)
     # maybe closing a connection to the host is required...
     try:
         instance.close()
@@ -260,5 +281,22 @@ def generator(config=None, transport=None, formatter=None, gap=None,
         lgr.warning('connection closing not implemented for chosen transport.')
 
 
+def list_fake_types():
+    """prints a list of random data types with an example"""
+    fake = Factory.create()
+    ignore_list = [
+        'add_provider',
+        'format',
+        'get_formatter',
+        'set_formatter',
+        'parse',
+        'provider',
+        'providers'
+    ]
+    for fake_type in dir(fake):
+        if not fake_type.startswith('_') and not fake_type in ignore_list:
+            print('*** {} ({})'.format(fake_type, getattr(fake, fake_type)()))
+
+
 class PylogError(Exception):
     pass
